import idna

from OpenSSL import SSL
from OpenSSL.SSL import SysCallError, Error
from cryptography import x509
from retry import retry
from socket import socket

from si_tests import logger
from si_tests.utils import waiters

LOG = logger.logger


@retry((SysCallError, Error), delay=60, tries=3, logger=LOG)
def get_certificate_from_host(hostname, port=443):
    hostname_idna = idna.encode(hostname)
    sock = socket()

    sock.connect((hostname, port))
    ctx = SSL.Context(SSL.SSLv23_METHOD)
    ctx.check_hostname = False
    ctx.verify_mode = SSL.VERIFY_NONE

    sock_ssl = SSL.Connection(ctx, sock)
    sock_ssl.set_connect_state()
    sock_ssl.set_tlsext_host_name(hostname_idna)
    sock_ssl.do_handshake()
    cert = sock_ssl.get_peer_certificate()
    crypto_cert = cert.to_cryptography()
    sock_ssl.close()
    sock.close()

    return crypto_cert


def app_to_tlsspec(app):
    temp = app.split('-')
    tlsSpec = temp[0] + ''.join(ele.title() for ele in temp[1:])
    tlsSpec = tlsSpec.replace("Alertmanager", "AlertManager")
    return tlsSpec


def wait_for_cert_applied(cert_pem, hostname):
    x509_cert = x509.load_pem_x509_certificate(cert_pem.encode())
    waiters.wait(
        lambda: get_certificate_from_host(hostname) == x509_cert,
        interval=10, timeout=600,
        timeout_msg="Certificate not changed")


class CertManager:
    def __init__(self, kaas_manager):
        self._kaas_manager = kaas_manager

    @property
    def kaas_manager(self):
        return self._kaas_manager

    def apply_cert_for_app(self, app, cluster, hostname,
                           cert_pem, key_pem, ca_pem=None):
        tlsconfig = self.kaas_manager.create_tlsconfig(
            name=f"{app}-{cluster.name}", namespace=cluster.namespace,
            hostname=hostname, cert_pem=cert_pem, key_pem=key_pem, ca_pem=ca_pem)
        cluster.update_tlsconfigref_for_app(app=app_to_tlsspec(app), tlsconfigref=tlsconfig.name)

        wait_for_cert_applied(cert_pem, hostname)
